@@ -7,16 +7,17 @@
 #include "base/android/jni_android.h"
 
 // Must come after all headers that specialize FromJniType() / ToJniType().
-#include "device/bluetooth/jni_headers/Wrappers_jni.h"
+#include "device/bluetooth/jni_headers/BluetoothAdapterWrapper_jni.h"
 
 using base::android::AttachCurrentThread;
 using base::android::ScopedJavaLocalRef;
 
 namespace device {
 
-ScopedJavaLocalRef<jobject> BluetoothAdapterWrapper_CreateWithDefaultAdapter() {
+ScopedJavaLocalRef<jobject> BluetoothAdapterWrapper_CreateWithDefaultAdapter(
+    bool enable_classic) {
   return Java_BluetoothAdapterWrapper_createWithDefaultAdapter(
-      AttachCurrentThread());
+      AttachCurrentThread(), enable_classic);
 }
 
 }  // namespace device
