@@ -91,8 +91,10 @@ class HeadlessVideoOverlayWindow : public content::VideoOverlayWindow {
   void ShowInactive() override {}
   void Hide() override {}
   bool IsVisible() const override { return false; }
-  gfx::Rect GetBounds() override { return gfx::Rect(); }
-  void UpdateNaturalSize(const gfx::Size& natural_size) override {}
+  gfx::Rect GetBounds() override { return gfx::Rect(size_); }
+  void UpdateNaturalSize(const gfx::Size& natural_size) override {
+    size_ = natural_size;
+  }
   void SetPlaybackState(PlaybackState playback_state) override {}
   void SetPlayPauseButtonVisibility(bool is_visible) override {}
   void SetSkipAdButtonVisibility(bool is_visible) override {}
@@ -105,8 +107,15 @@ class HeadlessVideoOverlayWindow : public content::VideoOverlayWindow {
   void SetHangUpButtonVisibility(bool is_visible) override {}
   void SetNextSlideButtonVisibility(bool is_visible) override {}
   void SetPreviousSlideButtonVisibility(bool is_visible) override {}
+  void SetMediaPosition(const media_session::MediaPosition&) override {}
+  void SetSourceTitle(const std::u16string& source_title) override {}
+  void SetFaviconImages(
+      const std::vector<media_session::MediaImage>& images) override {}
 
   void SetSurfaceId(const viz::SurfaceId& surface_id) override {}
+
+ private:
+  gfx::Size size_;
 };
 
 }  // namespace
@@ -149,10 +158,17 @@ HeadlessContentBrowserClient::CreateBrowserMainParts(
   return std::make_unique<HeadlessBrowserMainParts>(*browser_);
 }
 
-void HeadlessContentBrowserClient::OverrideWebkitPrefs(
+void HeadlessContentBrowserClient::OverrideWebPreferences(
     content::WebContents* web_contents,
+    content::SiteInstance& main_frame_site,
     blink::web_pref::WebPreferences* prefs) {
   prefs->lazy_load_enabled = browser_->options()->lazy_load_enabled;
+
+  if (base::CommandLine::ForCurrentProcess()->HasSwitch(
+          ::switches::kForceHighContrast)) {
+    prefs->in_forced_colors = true;
+    prefs->preferred_contrast = blink::mojom::PreferredContrast::kMore;
+  }
 }
 
 void HeadlessContentBrowserClient::RegisterBrowserInterfaceBindersForFrame(
@@ -308,11 +324,9 @@ base::OnceClosure HeadlessContentBrowserClient::SelectClientCertificate(
 }
 
 bool HeadlessContentBrowserClient::ShouldEnableStrictSiteIsolation() {
-  // TODO(lukasza): https://crbug.com/869494: Instead of overriding
-  // ShouldEnableStrictSiteIsolation, //headless should inherit the default
-  // site-per-process setting from //content - this way tools (tests, but also
-  // production cases like screenshot or pdf generation) based on //headless
-  // will use a mode that is actually shipping in Chrome.
+  // Use --site-per-process as the only source of truth for enabling site
+  // isolation, see SiteIsolationPolicy::UseDedicatedProcessesForAllSites()
+  // in content/public/browser/site_isolation_policy.cc.
   return false;
 }
 
@@ -323,6 +337,7 @@ bool HeadlessContentBrowserClient::
 }
 
 bool HeadlessContentBrowserClient::IsInterestGroupAPIAllowed(
+    content::BrowserContext* browser_context,
     content::RenderFrameHost* render_frame_host,
     content::InterestGroupApiOperation operation,
     const url::Origin& top_frame_origin,
@@ -333,8 +348,7 @@ bool HeadlessContentBrowserClient::IsInterestGroupAPIAllowed(
 bool HeadlessContentBrowserClient::IsPrivacySandboxReportingDestinationAttested(
     content::BrowserContext* browser_context,
     const url::Origin& destination_origin,
-    content::PrivacySandboxInvokingAPI invoking_api,
-    bool post_impression_reporting) {
+    content::PrivacySandboxInvokingAPI invoking_api) {
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
   return command_line->HasSwitch(switches::kForceReportingDestinationAttested);
 }
@@ -358,6 +372,26 @@ bool HeadlessContentBrowserClient::IsSharedStorageSelectURLAllowed(
   return true;
 }
 
+bool HeadlessContentBrowserClient::IsFencedStorageReadAllowed(
+    content::BrowserContext* browser_context,
+    content::RenderFrameHost* rfh,
+    const url::Origin& top_frame_origin,
+    const url::Origin& accessing_origin) {
+  return true;
+}
+
+bool HeadlessContentBrowserClient::IsCookieDeprecationLabelAllowed(
+    content::BrowserContext* browser_context) {
+  return true;
+}
+
+bool HeadlessContentBrowserClient::IsCookieDeprecationLabelAllowedForContext(
+    content::BrowserContext* browser_context,
+    const url::Origin& top_frame_origin,
+    const url::Origin& context_origin) {
+  return true;
+}
+
 void HeadlessContentBrowserClient::ConfigureNetworkContextParams(
     content::BrowserContext* context,
     bool in_memory,
@@ -453,6 +487,12 @@ HeadlessContentBrowserClient::CreateWindowForVideoPictureInPicture(
   return std::make_unique<HeadlessVideoOverlayWindow>();
 }
 
+// TODO(364362654, 40052246): force-disable network service sandboxing
+// until it's stable in headful.
+bool HeadlessContentBrowserClient::ShouldSandboxNetworkService() {
+  return false;
+}
+
 void HeadlessContentBrowserClient::HandleExplicitlyAllowedPorts(
     ::network::mojom::NetworkService* network_service) {
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
