@@ -203,11 +203,29 @@ class Attribute(NodeBase):
 
 
 class AttributeList(NodeListBase):
-  """Represents a list attributes."""
+  """Represents a list of attributes."""
 
   _list_item_type = Attribute
 
 
+class AttributeValueOrList(NodeListBase):
+  """Represents a list of pipe delimited attribute values."""
+
+  def __str__(self):
+    return '|'.join(item.name for item in self)
+
+  _list_item_type = Name
+
+
+class AttributeValueAndList(NodeListBase):
+  """Represents a list of ampersand delimited attribute values."""
+
+  def __str__(self):
+    return '&'.join(item.name for item in self)
+
+  _list_item_type = Name
+
+
 class Const(Definition):
   """Represents a const definition."""
 
@@ -226,6 +244,11 @@ class Const(Definition):
            self.typename == other.typename and \
            self.value == other.value
 
+  def __repr__(self):
+    return "Const(typename=%s, attribute_list=%s, value=%s)" % (
+        self.typename, self.attribute_list, self.value)
+
+
 
 class Enum(Definition):
   """Represents an enum definition."""
