@@ -2,15 +2,13 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#ifdef UNSAFE_BUFFERS_BUILD
-// TODO(crbug.com/351564777): Remove this and convert code to safer constructs.
-#pragma allow_unsafe_buffers
-#endif
-
 #include "services/proxy_resolver/proxy_resolver_v8.h"
 
+#include <array>
+
 #include "base/compiler_specific.h"
 #include "base/files/file_util.h"
+#include "base/notreached.h"
 #include "base/path_service.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
@@ -77,8 +75,7 @@ class MockJSBindings : public ProxyResolverV8::JSBindings {
       return !dns_resolve_ex_result.empty();
     }
 
-    CHECK(false);
-    return false;
+    NOTREACHED();
   }
 
   void OnError(int line_number, const std::u16string& message) override {
@@ -211,11 +208,14 @@ TEST_F(ProxyResolverV8Test, BadReturnType) {
   // These are the filenames of PAC scripts which each return a non-string
   // types for FindProxyForURL(). They should all fail with
   // net::ERR_PAC_SCRIPT_FAILED.
-  static const char* const filenames[] = {
-      "return_undefined.js", "return_integer.js", "return_function.js",
+  static const auto filenames = std::to_array<const char*>({
+      "return_undefined.js",
+      "return_integer.js",
+      "return_function.js",
       "return_object.js",
       // TODO(eroman): Should 'null' be considered equivalent to "DIRECT" ?
-      "return_null.js"};
+      "return_null.js",
+  });
 
   for (size_t i = 0; i < std::size(filenames); ++i) {
     ASSERT_THAT(CreateResolver(filenames[i]), IsOk());
@@ -559,5 +559,19 @@ TEST_F(ProxyResolverV8Test, Terminate) {
   EXPECT_EQ("[kittens:88]", proxy_info.proxy_chain().ToDebugString());
 }
 
+TEST_F(ProxyResolverV8Test, NoWebAssembly) {
+  ASSERT_THAT(CreateResolver("no_webassembly.js"), IsOk());
+
+  net::ProxyInfo proxy_info;
+  int result = resolver().GetProxyForURL(GURL("http://www.google.com"),
+                                         &proxy_info, bindings());
+
+  EXPECT_THAT(result, IsOk());
+  EXPECT_TRUE(proxy_info.is_direct());
+
+  EXPECT_EQ(0U, bindings()->alerts.size());
+  EXPECT_EQ(0U, bindings()->errors.size());
+}
+
 }  // namespace
 }  // namespace proxy_resolver
